
import SwiftUI
import QuickLook
import Foundation
import AppKit

@main

struct rotaterApp: App {
    @Environment(\.openURL) var openURL
    let helpWindowController = HelpWindowController()
    @AppStorage("DefaultWindowWidth") var DefaultWindowWidth = 1000
    @AppStorage("DefaultWindowHeight") var DefaultWindowHeight = 500
    
    init() {
        UserDefaults.standard.set(true, forKey: "NSDisabledDictationMenuItem")
        UserDefaults.standard.set(true, forKey: "NSDisabledCharacterPaletteMenuItem")
        UserDefaults.standard.set(true, forKey: "NSFullScreenMenuItemEverywhere")
    }
    
    var body: some Scene {
        DocumentGroup(newDocument: rotaterDocument()) { file in MetalView(thetext: file.$document.text) }
        
        // Better but breaks New menu
        //DocumentGroup(viewing: rotaterDocument.self) { file in MetalView(thetext: file.$document.text) }
        
        // defaultSize only available in macOS 13+
            .CUSTOMdefaultSize(width: CGFloat(DefaultWindowWidth), height: CGFloat(DefaultWindowHeight+28))
        
        
            .commands {
                
                // Take away save menu items - but breaks Open Recent in MacOS 14 and 15
                //CommandGroup(replacing: CommandGroupPlacement.saveItem) {
                //    EmptyView()
                //}
                
                
                // Replace help menu
                CommandGroup(replacing: .help) {
                    Button("Rotater Help") { helpWindowController.showWindow(nil) }
                    Divider()
                    Button("Rotater Web Site") { openURL(URL(string: "https://treefish.com/rotater/")!) }
                }
                
                
                // Put check for update menu item into App menu
                CommandGroup(after: .appInfo) {
                    Divider()
                    Button("Check for Updates…", action: {
                        Task {
                            var releaseAppVersion = await getLatestVersionNumber()
                            if releaseAppVersion.count > 10 { releaseAppVersion = "" }
                            
                            let thisAppVersion = Bundle.main.object(forInfoDictionaryKey: "CFBundleShortVersionString") as? String ?? ""
                            
                            let alert = NSAlert()
                            alert.addButton(withTitle: "OK")
                            
                            if thisAppVersion == "" || releaseAppVersion == "" {
                                alert.messageText = "Could not check Rotater version"
                                alert.informativeText = "You can try going to the web site"
                                alert.addButton(withTitle: "Go to web site")
                            }
                            else if thisAppVersion == releaseAppVersion {
                                alert.messageText = "Rotater is up to date"
                                alert.informativeText = "You are running the latest available version (" + thisAppVersion + ")"
                            }
                            else {
                                alert.messageText = "A new version of Rotater is available"
                                alert.informativeText = "You are running " + thisAppVersion + " while " + releaseAppVersion + " is available on the web site"
                                alert.addButton(withTitle: "Go to web site")
                            }
                            
                            if alert.runModal() == .alertSecondButtonReturn { openURL(URL(string: "https://treefish.com/rotater/")!) }
                        }
                    })
                }
                
                
                // Remove undo and redo menu items
                CommandGroup(replacing: .undoRedo) { }
                
                
                // Add a new Action menu
                CommandMenu("Action") {
                    Group {
                        Button("Start/Stop Rotation") { sendKeyPress(49, false) }
                            .keyboardShortcut(" ", modifiers: [])
                        
                        Divider()
                        
                        Button("Zoom In") { sendKeyPress(24, false) }
                            .keyboardShortcut("=", modifiers: [])
                        
                        Button("Zoom Out") { sendKeyPress(27, false) }
                            .keyboardShortcut("-", modifiers: [])
                        
                        Divider()
                    }
                    
                    Group {
                        Button("Thinner Lines") { sendKeyPress(43, false) }
                            .keyboardShortcut(",", modifiers: [])
                        
                        Button("Thicker Lines") { sendKeyPress(47, false) }
                            .keyboardShortcut(".", modifiers: [])
                        
                        Divider()
                        
                        Button("Smaller Points") { sendKeyPress(43, true) }
                            .keyboardShortcut("<", modifiers: [])
                        
                        Button("Bigger Points") { sendKeyPress(47, true) }
                            .keyboardShortcut(">", modifiers: [])
                        
                        Divider()
                    }
                    
                    Group {
                        Button("Centering") { sendKeyPress(8, false) }
                            .keyboardShortcut("c", modifiers: [])
                        
                        Button("Handedness") { sendKeyPress(4, false) }
                            .keyboardShortcut("h", modifiers: [])
                        
                        Menu("Direction") {
                            Button("X Up") { sendKeyPress(7, false) }
                                .keyboardShortcut("x", modifiers: [])
                            
                            Button("Y Up") { sendKeyPress(16, false) }
                                .keyboardShortcut("y", modifiers: [])
                            
                            Button("Z Up") { sendKeyPress(6, false) }
                                .keyboardShortcut("z", modifiers: [])
                            
                            Button("X Down") { sendKeyPress(7, true) }
                                .keyboardShortcut("x", modifiers: [.shift])
                            
                            Button("Y Down") { sendKeyPress(16, true) }
                                .keyboardShortcut("y", modifiers: [.shift])
                            
                            Button("Z Down") { sendKeyPress(6, true) }
                                .keyboardShortcut("z", modifiers: [.shift])
                        }
                        
                        Divider()
                        
                        Button("Rotation Mode") { sendKeyPress(17, false) }
                            .keyboardShortcut("t", modifiers: [])
                    }
                    
                    Group {
                        Button("Stereo Mode") { sendKeyPress(1, false) }
                            .keyboardShortcut("s", modifiers: [])
                        
                        Button("Decrease Stereo Angle") { sendKeyPress(18, false) }
                            .keyboardShortcut("1", modifiers: [])
                        
                        Button("Increase Stereo Angle") { sendKeyPress(19, false) }
                            .keyboardShortcut("2", modifiers: [])
                        
                        Divider()
                    }
                    
                    Group {
                        Button("Background Color") { sendKeyPress(11, false) }
                            .keyboardShortcut("b", modifiers: [])
                        
                        Button("Frames Per Second") { sendKeyPress(3, false) }
                            .keyboardShortcut("f", modifiers: [])
                        
                        Button("Wireframe Mode") { sendKeyPress(13, false) }
                            .keyboardShortcut("w", modifiers: [])
                        
                        Button("Window Size") { sendKeyPress(46, false) }
                            .keyboardShortcut("m", modifiers: [])
                        
                        Divider()
                    }
                    
                    Group {
                        Button("Reset Angles") { sendKeyPress(15, false) }
                            .keyboardShortcut("r", modifiers: [])
                        
                        Button("Reset All Settings") { sendKeyPress(15, true) }
                            .keyboardShortcut("R", modifiers: [.shift])
                        
                        Button("Save Default Settings") { sendKeyPress(2, true) }
                            .keyboardShortcut("D", modifiers: [.shift])
                        
                        Divider()
                        
                        Button("In Window Help") { sendKeyPress(34, false) }
                            .keyboardShortcut("i", modifiers: [])
                    }
                }
            }
    }
    
    
    // Send the supplied key down event to the front window
    func sendKeyPress(_ keyCode: UInt16, _ shiftSet: Bool) {
        let winID = NSApplication.shared.keyWindow
        let winNum = winID?.windowNumber
        if winNum != nil {
            var theFlags = NSEvent.ModifierFlags.shift
            if !shiftSet { theFlags = [] }
            let a = NSEvent.keyEvent(with: .keyDown, location: NSPoint(x: 0, y: 0), modifierFlags: theFlags, timestamp: 0, windowNumber: winNum!, context: nil, characters: "a", charactersIgnoringModifiers: "a", isARepeat: false, keyCode: keyCode)
            winID!.postEvent(a!, atStart: true)
        }
    }
    
    
    // Check for updates and show alert
    func getLatestVersionNumber() async -> String {
        // Get latest release version from Rotater web site
        guard let checkVersionURL = URL(string: "https://treefish.com/rotater/rotatercurrentversion.txt")
        else { return "Bad URL being requested" }
        do {
            let session = URLSession(configuration: .ephemeral) // no cacheing
            let (releaseAppVersionReturn, _) = try await session.data(from: checkVersionURL)
            return String(data: releaseAppVersionReturn, encoding: .utf8) ?? "Version not returned"
        }
        catch { return "No valid data was returned" }
    }
    
}


// Do the window the old way for macOS 11 compatability
class HelpWindowController: NSWindowController {
    convenience init() {
        let contentView = HelpWindowView()
        let window = NSWindow(
            // Make big enough to encase text, small enough to center on screen
            contentRect: NSRect(x: 0, y: 0, width: 250, height: 570),
            styleMask: [.titled, .closable, .miniaturizable, .fullSizeContentView],
            backing: .buffered,
            defer: false
        )
        window.center()
        window.title = "Rotater Help"
        window.contentView = NSHostingView(rootView: contentView)
        self.init(window: window)
    }
}


struct HelpWindowView: View {
    @State var rotaterManualURL: URL?
    var body: some View {
        ZStack {
            VStack(alignment: .leading) {
                
                VStack(alignment: .leading) {
                    Text("Keyboard Actions")
                }
                .font(.system(size: 14) .bold())
                .padding(2)
                
                HStack {
                    Text(infoHelpKeys)
                        .font(.system(size: 14))
                        .padding(2)
                    Text(infoHelpText)
                        .font(.system(size: 14))
                        .padding(2)
                }
                .padding(0)
                
                VStack(alignment: .center) {
                    Button("User Manual", action: {
                        rotaterManualURL = Bundle.main.url(forResource: "RotaterManual", withExtension: "pdf")
                    })
                    .keyboardShortcut(.defaultAction)
                    .quickLookPreview($rotaterManualURL)
                }
                .padding(0)
            }
            .padding(8)
        }
    }
}
