
import SwiftUI
import UniformTypeIdentifiers
import Foundation

extension UTType {
    static var importText: UTType { UTType(importedAs: "com.kloeden.rotater.rot") }
    static var rotaterText: UTType { UTType(filenameExtension: "rot")! }
}


struct rotaterDocument: FileDocument {
    // Read in pretty much any file type
    static var readableContentTypes: [UTType] { [.importText, .rotaterText, .text, .data, .item] }
    var text: String
    
    // New document content
    init(text: String = "0 0 0 -7\n1 1 1 -7\n-1 1 1 1\n-1 1 -1 3\n1 1 -1 1\n1 1 1 3\n1 -1 1 -7\n-1 -1 1 1\n-1 -1 -1 3\n1 -1 -1 1\n1 -1 1 3\n1 1 1 -7\n1 -1 1 2\n-1 1 1 -7\n-1 -1 1 2\n1 1 -1 -7\n1 -1 -1 2\n-1 1 -1 -7\n-1 -1 -1 2\n-1 -1 1 -7\n1 -1 -1 -7\n-1 -1 -1 -7\n0 0 0 0\n0.5 0 0 1\n0 0 0 0\n0 0.5 0 2\n0 0 0 0\n0 0 0.5 3\n") {
        self.text = text
    }
    
    // Read files as isoLatin1 to avoid dealing with UTF and ASCII errors
    init(configuration: ReadConfiguration) throws {
        guard let data = configuration.file.regularFileContents,
              let string = String(data: data, encoding: .isoLatin1) // .ascii fails on odd characters
        else { throw CocoaError(.fileReadCorruptFile) }
        text = string
    }
    
    // Disallow writing file - throw an error
    func fileWrapper(configuration: WriteConfiguration) throws -> FileWrapper {
        //let data = text.data(using: .isoLatin1)!
        //return .init(regularFileWithContents: data)
        
        // Just disallow writing
        throw CocoaError(.fileWriteUnknown)
    }
}
